@@ -301,7 +301,7 @@ repo_new(JSContext *context)
     repo = JS_NewObject(context, &gjs_repo_class, NULL, global);
     if (repo == NULL) {
         gjs_throw(context, "No memory to create repo object");
-        return JS_FALSE;
+        return NULL;
     }
 
     priv = g_slice_new0(Repo);
@@ -530,7 +530,9 @@ gjs_define_info(JSContext  *context,
             return JS_FALSE;
         break;
     case GI_INFO_TYPE_INTERFACE:
-        gjs_define_interface_class(context, in_object, (GIInterfaceInfo*) info);
+        gjs_define_interface_class(context, in_object, (GIInterfaceInfo *) info,
+                                   g_registered_type_info_get_g_type((GIRegisteredTypeInfo *) info),
+                                   NULL);
         break;
     default:
         gjs_throw(context, "API of type %s not implemented, cannot define %s.%s",
